#!/usr/bin/perl -w

# LSB Test Execution Framework
# Configuration Save Module (tests_conf_save.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use Fcntl ':flock';
use Fcntl ':seek';

autoflush_on();

sub is_process_finished($) {
	my $PID = $_[0];
	my $test_line = `ps -p $PID -o s --no-heading`;
	$test_line =~ s/[\r\n]//g;
	return ($test_line !~ m/^[^Zz]/);
}

# Helps to determine platform name.
sub guess_platform($) {
    if ($_[0] =~ /^(amd64|x86_64)$/) {
		return "x86_64";
	}
    elsif ($_[0] =~ /^(ia32|i[3-6]86|x86)$/) {
		return "ia32";
	}
    elsif ($_[0] =~ /^(ia64)$/) {
		return "ia64";
	}
    elsif ($_[0] =~ /^(ppc|ppc32)$/) {
		return "ppc32";
	}
    elsif ($_[0] =~ /^(ppc64)$/) {
		return "ppc64";
	}
    elsif ($_[0] =~ /^(s390)$/) {
		return "s390";
	}
    elsif ($_[0] =~ /^(s390x)$/) {
		return "s390x";
	}
    else {
		return "unknown";
	}
}

# Adds directory contents to the list (recursively)
sub add_dir_to_list($$) {
	my ($dir, $file_list) = @_;
	if (opendir(DIR, $dir)) {
		foreach (sort(grep(!/^\.+$/, readdir(DIR)))) {
			if (-f "$dir/$_") {
				$file_list->{"$dir/$_"} = 1;
			}
			else {
				&add_dir_to_list("$dir/$_", $file_list);
			}
		}
		closedir(DIR);
	}
}

# Creates the INFO file for a particular journal and fills it with the information needed
sub create_app_journal_info($$$$$) {
	my ($jpath, $fpath, $path_root_real, $path_root_show, $no_check) = @_;
	my $ftype = `file $fpath`;
	if ($no_check or ($ftype =~ m/executable/) or ($ftype =~ m/shared object/)) {
		my $fname = $fpath;
		$fname =~ s|^.*/([^/]*)$|$1|;
		my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($fpath);
		if (open(INFO_FILE, ">$jpath/lsbappchk.$fname.$dev-$ino.journal.INFO")) {
			$fpath =~ s/^$path_root_real/$path_root_show\|/;
			print INFO_FILE "TITLE: $fpath\nANCHOR: lsbappchk_${fname}_${dev}_$ino\nJOURNAL_FILENAME: lsbappchk.$fname.$dev-$ino.journal\n";
			close(INFO_FILE);
		}
	}
}

sub create_pkg_journal_info($$) {
	my ($jpath, $fpath) = @_;
	my $fname = $fpath;
	$fname =~ s|^.*/([^/]*)$|$1|;
	my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($fpath);
	if (open(INFO_FILE, ">$jpath/lsbpkgchk.$fname.$dev-$ino.journal.INFO")) {
		$fpath =~ s/$/\|/;
		print INFO_FILE "TITLE: $fpath\nANCHOR: lsbpkgchk_${fname}_${dev}_$ino\nJOURNAL_FILENAME: lsbpkgchk.$fname.$dev-$ino.journal\n";
		close(INFO_FILE);
	}
	return "$jpath/lsbpkgchk.$fname.$dev-$ino.journal";
}

my $conf_page = "tests_conf.pl";
if ($_GET{"goto"}) {
	$conf_page = $_GET{"goto"};
	$conf_page =~ s/%23/#/;
	$_POST{"form_action"} = "";
}
my $session_profile = ($_COOKIE{"session_id"} ? "~session.$PeerIP.".$_COOKIE{"session_id"} : "");

my $error;
my $new_url = "";
my $save_profile_name = "";

if (!-d "$APP_DATA/profiles") {
	system("mkdir -p $APP_DATA/profiles");
	if (!-d "$APP_DATA/profiles") {
		$error = "";
	}
}

if (!defined($error)) {
	if ((!$_POST{"form_action"} or ($_POST{"form_action"} eq "")) and (!$_GET{"goto"})) {
		$error = "Script action not specified!";
	}
	elsif ($_POST{"form_action"} eq "load_profile") {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid profile name!";
		}
		elsif (!-f "$APP_DATA/profiles/".$_POST{"edit_profile_name"}) {
			$error = "File not found:<br />$APP_DATA/profiles/".$_POST{"edit_profile_name"};
		}
		else {
			system("cp \"$APP_DATA/profiles/$_POST{edit_profile_name}\" \"$APP_DATA/profiles/$session_profile\" > /dev/null");
			$new_url = $conf_page;
		}
	}
	elsif ($_POST{"form_action"} eq "run_tests") {
		if ($session_profile eq "") {
			$error = "Cannot retrieve session ID! Please, allow cookies in your browser.";
		}
		else {
			$save_profile_name = $session_profile;
			$new_url = "tests_results.pl";
		}
	}
	elsif ($_POST{"form_action"} eq "save_profile") {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid profile name!";
		}
		else {
			$save_profile_name = $_POST{"edit_profile_name"};
			$new_url = $conf_page;
		}
	}
	elsif ($_POST{"form_action"} eq "delete_profile") {
		if (!$_POST{"edit_profile_name"}) {
			$error = "No profile name specified!";
		}
		elsif ($_POST{"edit_profile_name"} =~ m/[\\\/]/) {
			$error = "Invalid profile name!";
		}
		elsif (!-f "$APP_DATA/profiles/".$_POST{"edit_profile_name"}) {
			$error = "File not found:<br />$APP_DATA/profiles/".$_POST{"edit_profile_name"};
		}
		else {
			unlink("$APP_DATA/profiles/".$_POST{"edit_profile_name"});
			$save_profile_name = $session_profile;
			$new_url = $conf_page;
		}
	}
	elsif ($_GET{"goto"}) {
		$save_profile_name = $session_profile;
		$new_url = $conf_page;
	}
}

if (!defined($error)) {
	if ($save_profile_name ne "") {
		my $full_save_name = "$APP_DATA/profiles/$save_profile_name";
		if (open(FILE, "> $full_save_name")) {
			my $architecture = ($_POST{"architecture"} or guess_platform(`uname -m`));
			my $lsb_version  = ($_POST{"lsb_version"} or "3.1");
			my $lsb_profile  = ($_POST{"lsb_profile"} or "desktop");
			print FILE "APPLICATION:     ".$_POST{"application"}."\n" if ($_POST{"application"});
			if ($_POST{"paths"}) {
				$_POST{"paths"} =~ s/\r\n/\n/g;
				my @paths_lines = split(/\n/, $_POST{"paths"});
				foreach (@paths_lines) {
					print FILE "PATH:            $_\n";
				}
			}
			if ($_POST{"packages"}) {
				$_POST{"packages"} =~ s/\r\n/\n/g;
				my @packages_lines = split(/\n/, $_POST{"packages"});
				foreach (@packages_lines) {
					print FILE "PACKAGE:         $_\n";
				}
			}
			print FILE "ARCHITECTURE:    $architecture\n";
			print FILE "LSB_VERSION:     $lsb_version\n";
			print FILE "LSB_PROFILE:     $lsb_profile\n";

			if ($_POST{"comments"}) {
				$_POST{"comments"} =~ s/\r\n/\n/g;
				$_POST{"comments"} =~ s/&/&amp;/g;
				$_POST{"comments"} =~ s/</&lt;/g;
				$_POST{"comments"} =~ s/>/&gt;/g;
				my @comments_lines = split(/\n/, $_POST{"comments"});
				foreach (@comments_lines) {
					print FILE "COMMENTS: $_\n";
				}
			}

			close(FILE);

			if ($_POST{"form_action"} eq "save_profile") {
				system("cp \"$full_save_name\" \"$APP_DATA/profiles/$session_profile\" > /dev/null");
			}
			elsif ($_POST{"form_action"} eq "run_tests") {
				if (!$_POST{"application"}) {
					$error = "Application name is not specified.";
				}
				else {
					print "HTTP/1.0 200 OK\n";
					print "Content-type: text/html\n\n";
					print_header("Test Execution", "conf");
					
					# Show timer with waiting message
					print <<DATA;
<div id="progress_info">
<font color="green"><b>Please, wait while the test is running...</b></font><br />
Time elapsed: <span id="timer" style="font-family: monospace; color: blue;">00:00</span></p>
</div>

<script language="javascript" type="text/javascript">
// <![CDATA[
var elapsed_time = 0;
var finished = 0;
function update_timer() {
	++elapsed_time;
	var elem = document.getElementById("timer");
	if (elem && !finished) {
		var m = Math.floor(elapsed_time / 60);
		var s = elapsed_time - m * 60;
		var tmp = "";
		tmp += (m < 10) ? "0" + m : m;
		tmp += ":";
		tmp += (s < 10) ? "0" + s : s;
		elem.innerHTML = tmp;
		setTimeout("update_timer()", 1000);
	}
}

setTimeout("update_timer()", 1000);
// ]]>
</script>
DATA
					print_footer(1);

					my $app_name = $_POST{"application"};

					# Create results directory
					my @clean_dirs = ();
					my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
					my $host_name = `uname -n`;
					$host_name =~ s/[\r\n]//g;
					my $results_dir = sprintf("$architecture-$host_name-$app_name-%04d-%02d-%02d-%02dh-%02dm-%02ds", ($year + 1900), ($mon + 1), $mday, $hour, $min, $sec);
					my $results_dir_esc = $results_dir;
					$results_dir_esc =~ s/ /\\ /g;
					system("mkdir -p ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc/test_journals");
					system("mkdir -p ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc/test_logs");
					if ((!-d $CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals") or (!-d $CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs")) {
						$error = "Could not create subdirectories for storing the test results in<br />$results_dir<br />$!";
					}
					else {
						# Prepare data for testing
						my $tests_output = $CONFIG{"RESULTS_DIR"}."/$results_dir_esc/test_logs/tests_output";
						my $cmd_line = $CONFIG{"TESTS_DIR"}."/lsbappchk -r $lsb_version -j -o ".$CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals -l /tmp/$app_name.list.$$ ";
						if ($lsb_profile eq "desktop") {
							$cmd_line .= "-Tcore,c++,desktop";
						}
						elsif ($lsb_profile eq "core") {
							$cmd_line .= "-Tcore,c++";
						}
						$idx = 0;
						my %file_list;
						my @failed_files = ();
						my @rpm_files = ();
						if (open(LIST_FILE, ">/tmp/$app_name.list.$$")) {
							if (open(LIST_FILE_DB, ">/tmp/$app_name.list_db.$$")) {
								if ($_POST{"paths"}) {
									my @paths_lines = split(/\n/, $_POST{"paths"});
									foreach my $path (@paths_lines) {
										if (-d $path) {
											%file_list = ();
											add_dir_to_list($path, \%file_list);
											foreach (sort keys %file_list) {
												create_app_journal_info($CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals", $_, $path, $path, 0);
												print LIST_FILE "$_\n";
												print LIST_FILE_DB "$_\n";
											}
										}
										elsif ($path =~ m/\.rpm$/) {
											system("echo \"Unpacking RPM package $path\" >>$tests_output");
											$idx++;
											mkdir("/tmp/pkg-unpack.$$-$idx");
											push @clean_dirs, "/tmp/pkg-unpack.$$-$idx";
											my $path_esc = $path;
											$path_esc =~ s/ /\\ /g;
											system("cd /tmp/pkg-unpack.$$-$idx; rpm2cpio $path_esc 2>>$tests_output | cpio -id 2>>$tests_output");
											if (!$?) {
												my $j_path = create_pkg_journal_info($CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals", $path);
												push @rpm_files, $path_esc;
												push @rpm_journals, $j_path;
												print LIST_FILE_DB "$path\n";
												%file_list = ();
												add_dir_to_list("/tmp/pkg-unpack.$$-$idx", \%file_list);
												foreach (sort keys %file_list) {
													create_app_journal_info($CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals", $_, "/tmp/pkg-unpack.$$-$idx", $path, 0);
													print LIST_FILE "$_\n";
												}
											}
											else {
												push @failed_files, $path;
											}
										}
										elsif (($path =~ m/\.tgz$/) or ($path =~ m/\.tar\.gz$/)) {
											system("echo \"Unpacking TGZ package $path\" >>$tests_output");
											$idx++;
											mkdir("/tmp/pkg-unpack.$$-$idx");
											push @clean_dirs, "/tmp/pkg-unpack.$$-$idx";
											my $path_esc = $path;
											$path_esc =~ s/ /\\ /g;
											system("cd /tmp/pkg-unpack.$$-$idx; tar xzf $path_esc 2>>$tests_output");
											if (!$?) {
												print LIST_FILE_DB "$path\n";
												%file_list = ();
												add_dir_to_list("/tmp/pkg-unpack.$$-$idx", \%file_list);
												foreach (sort keys %file_list) {
													create_app_journal_info($CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals", $_, "/tmp/pkg-unpack.$$-$idx", $path, 0);
													print LIST_FILE "$_\n";
												}
											}
											else {
												push @failed_files, $path;
											}
										}
										else {
											create_app_journal_info($CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals", $path, "", "", 1);
											print LIST_FILE "$path\n";
											print LIST_FILE_DB "$path\n";
										}
									}
								}
								if ($_POST{"packages"}) {
									my @packages_lines = split(/\n/, $_POST{"packages"});
									foreach my $pkg (@packages_lines) {
										my $pkg_list = `rpm -ql $pkg`;
										foreach (split(/\n/, $pkg_list)) {
											if (!-d) {
												create_app_journal_info($CONFIG{"RESULTS_DIR"}."/$results_dir/test_journals", $_, "", "RPM: $pkg", 0);
												print LIST_FILE "$_\n";
												print LIST_FILE_DB "$_\n";
											}
										}
									}
								}
								close(LIST_FILE_DB);
							}
							close(LIST_FILE);
						}
						if (scalar(@failed_files) == 0) {
							for (my $i = 0; $i < scalar(@rpm_files); $i++) {
								my $cmd_line_pkg = $CONFIG{"TESTS_DIR"}."/lsbpkgchk -r $lsb_version -j ".$rpm_journals[$i];
								if ($lsb_profile eq "desktop") {
									$cmd_line_pkg .= " -Tcore,c++,desktop ";
								}
								elsif ($lsb_profile eq "core") {
									$cmd_line_pkg .= " -Tcore,c++ ";
								}
								$cmd_line_pkg .= $rpm_files[$i];
								system($cmd_line_pkg." >>$tests_output 2>&1");
							}
							system($cmd_line." >>$tests_output 2>&1");
							if (open(FILE, "+<".$CONFIG{"RESULTS_DIR"}."/HISTORY")) {
								flock(FILE, LOCK_EX);
								seek(FILE, 0, SEEK_END);
								print FILE "$results_dir\n";
								flock(FILE, LOCK_UN);
								close(FILE);
							}
							if (open(COMMENTS_FILE, ">$CONFIG{RESULTS_DIR}/$results_dir/test_logs/test_comments")) {
								print COMMENTS_FILE $_POST{"comments"};
								close(COMMENTS_FILE);
							}
							system("echo \"$lsb_version\" >> ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc/test_logs/lsb_version");
							system("echo \"$architecture\" >> ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc/test_logs/host_platform");

							# Now all preparations are done, generate the report
							if (-s $CONFIG{"RESULTS_DIR"}."/$results_dir/test_logs/tests_output" != 0) {
								system("echo \"\" >>$tests_output");
							}
							system($CONFIG{"UTILS_DIR"}."/report.pl ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc -n $app_name -o lsbappchk-$results_dir_esc.tgz >>$tests_output 2>&1");

							# Generate application analysis data
							# apptodb needs PM file located in its directory.
							chdir($CONFIG{"UTILS_DIR"});
							system("echo \"\" >>$tests_output");
							system("echo \"Collecting data about application files\" >>$tests_output");
							system($CONFIG{"UTILS_DIR"}."/apptodb.pl -a $architecture -l /tmp/$app_name.list_db.$$ -o ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc/${app_name}_${architecture}_upload_data $app_name >>$tests_output 2>&1");
							$new_url = "tests_results.pl?details=$results_dir";
						}
						else {
							$error = "Could not unpack the following files:<br />".join("<br />", @failed_files);
						}
						unlink("/tmp/$app_name.list.$$");
						unlink("/tmp/$app_name.list_db.$$");
						foreach (@clean_dirs) {
							system("rm -rf $_ >/dev/null 2>&1");
						}
					}
					if (defined($error)) {
						system("rm -rf ".$CONFIG{"RESULTS_DIR"}."/$results_dir_esc >/dev/null 2>&1");
						escape($error);
						print <<DATA;
<script language="javascript" type="text/javascript">
// <![CDATA[
finished = 1;
document.cookie="last_error_text=$error; path=/";
location.replace("$conf_page");
// ]]>
</script>
DATA
					}
					else {
						print <<DATA;
<script language="javascript" type="text/javascript">
// <![CDATA[
finished = 1;
location.replace("$new_url");
// ]]>
</script>
DATA
					}
					print "</body>\n</html>\n";
					exit;
				}
			}
		}
		else {
			$error = "Cannot open file $save_profile_name for writing!<br />$!";
		}
	}
}

if (defined($error)) {
	escape($error);
	print "HTTP/1.0 302 Moved\n";
	print "Set-Cookie: last_error_text=$error; path=/;\n";
	print "Location: $conf_page\n\n";
}
else {
	print "HTTP/1.0 302 Moved\n";
	print "Location: $new_url\n\n";
}
